@@ -1,7 +1,8 @@
 # emacs: -*- mode: python-mode; py-indent-offset: 2; tab-width: 2; indent-tabs-mode: nil -*-
 # ex: set sts=2 ts=2 sw=2 et:
 
-__all__ = ['dataset', 'mask', 'imageutils', 'mappable', 'transformations', 'lexparser']
+__all__ = ['dataset', 'mask', 'imageutils',
+           'mappable', 'transformations', 'lexparser']
 
 # Verify nibabel version right here since nothing would work
 import nibabel as nib
